# nested spartan methods
require('dplyr')
require('spartan')

lhc_constructcoeff_dataset = function (LHCRESULTFILE, PARAMNAME, PARAMETERS) 
{
  coeff_data <- NULL
  coeff_headers <- NULL
  for (m in 1:length(PARAMETERS)) {
    if (PARAMETERS[m] != PARAMNAME) {
      coeff_data <- cbind(coeff_data, LHCRESULTFILE[, PARAMETERS[m]])
      coeff_headers <- cbind(coeff_headers, PARAMETERS[m])
    }
    colnames(coeff_data) <- c(coeff_headers)
  }
  return(coeff_data)
}

pcor.test = function (x, y, z, use = "mat", calc_method = "p", na.rm = TRUE) 
{
  x <- c(x)
  y <- c(y)
  z <- as.data.frame(z, check.names = FALSE)
  useOk <- TRUE
  methodOk <- TRUE
  if (gregexpr("p", calc_method)[[1]][1] == 1) {
    p.method <- "Pearson"
  }
  else if (gregexpr("s", calc_method)[[1]][1] == 1) {
    p.method <- "Spearman"
  }
  else if (gregexpr("k", calc_method)[[1]][1] == 1) {
    p.method <- "Kendall"
  }
  else {
    message("'method' should be \"pearson\" or \"spearman\" or \"kendall\"!\n")
    methodOk <- FALSE
  }
  if (methodOk) {
    if (use == "mat") {
      p.use <- "Var-Cov matrix"
      pcor <- pcor.mat(x, y, z, cor_method = calc_method, 
                       na.rm = na.rm)
    }
    else if (use == "rec") {
      p.use <- "Recursive formula"
      pcor <- pcor.rec(x, y, z, cor_method = calc_method, 
                       na.rm = na.rm)
    }
    else {
      message("'use' should be either \"rec\" or \"mat\"!\n")
      useOk <- FALSE
    }
  }
  if (useOk && methodOk) {
    n <- dim(na.omit(data.frame(x, y, z, check.names = FALSE)))[1]
    gn <- dim(z)[2]
    if (p.method == "Kendall") {
      statistic <- pcor/sqrt(2 * (2 * (n - gn) + 5)/(9 * 
                                                       (n - gn) * (n - 1 - gn)))
      p.value <- 2 * pnorm(-abs(statistic))
    }
    else {
      val_to_sqrt_l <- n - 2 - gn
      val_to_sqrt_r <- 1 - pcor^2
      statistic <- pcor * sqrt(val_to_sqrt_l/val_to_sqrt_r)
      p.value <- 2 * pnorm(-abs(statistic))
    }
    data.frame(estimate = pcor, p.value = p.value, statistic = statistic, 
               n = n, gn = gn, Method = p.method, Use = p.use, check.names = FALSE)
  }
}

pcor.mat = function (x, y, z, cor_method = "p", na.rm = TRUE) 
{
  x <- c(x)
  y <- c(y)
  z <- as.data.frame(z, check.names = FALSE)
  if (dim(z)[2] == 0) {
    stop("There should be given data\n")
  }
  data <- data.frame(x, y, z, check.names = FALSE)
  if (na.rm == TRUE) {
    data <- na.omit(data)
  }
  xdata <- na.omit(data.frame(data[, c(1, 2)]), check.names = FALSE)
  Sxx <- cov(xdata, xdata, method = cor_method)
  xzdata <- na.omit(data)
  xdata <- data.frame(xzdata[, c(1, 2)], check.names = FALSE)
  zdata <- data.frame(xzdata[, -c(1, 2)], check.names = FALSE)
  Sxz <- cov(xdata, zdata, method = cor_method)
  zdata <- na.omit(data.frame(data[, -c(1, 2)]), check.names = FALSE)
  Szz <- cov(zdata, zdata, method = cor_method)
  zz.ev <- eigen(Szz)$values
  if (min(zz.ev)[1] < 0) {
    stop("'Szz' is not positive definite!\n")
  }
  Sxx.z <- Sxx - Sxz %*% solve(Szz) %*% t(Sxz)
  rxx.z <- cov2cor(Sxx.z)[1, 2]
  rxx.z
}

calculate_prcc_for_all_measures = function (MEASURES, COEFFPARAMCOL, COEFFDATA, LHCRESULTFILE, 
                                            cor_calc_method = c("s"), prcc_method = "mat") 
{
  PARAM_RESULTS <- NULL
  for (l in 1:length(MEASURES)) {
    COEFFMEASURERESULT <- LHCRESULTFILE[, MEASURES[l]]
    PARAMCOEFF <- pcor.test(COEFFPARAMCOL, COEFFMEASURERESULT, 
                            COEFFDATA, calc_method = cor_calc_method, use = prcc_method)
    if (!is.null(PARAMCOEFF)) 
      PARAM_RESULTS <- cbind(PARAM_RESULTS, PARAMCOEFF$estimate, 
                             PARAMCOEFF$p.value)
    else {
      message("Correlation Calculation method needs to be either s,p,or k, and prcc calculation method either rec or mat")
      return(NULL)
    }
  }
  return(PARAM_RESULTS)
}

generate_prcc_results_header = function (measures) 
{
  COEFFRESULTSHEAD <- NULL
  for (l in 1:length(measures)) {
    COEFFRESULTSHEAD <- cbind(COEFFRESULTSHEAD, (paste(measures[l], 
                                                       "_Estimate", sep = "")), (paste(measures[l], "_PValue", 
                                                                                       sep = "")))
  }
  return(COEFFRESULTSHEAD)
}


calculate_prccs_all_parameters = function (PARAMETERS, LHCRESULTFILE, MEASURES, cor_calc_method = c("s")) 
{
  COEFFRESULTS <- NULL
  for (k in 1:length(PARAMETERS)) {
    COEFFDATA <- lhc_constructcoeff_dataset(LHCRESULTFILE, 
                                            PARAMETERS[k], PARAMETERS)
    COEFFPARAMCOL <- LHCRESULTFILE[, PARAMETERS[k]]
    COEFFRESULTS <- rbind(COEFFRESULTS, calculate_prcc_for_all_measures(MEASURES, 
                                                                        COEFFPARAMCOL, COEFFDATA, LHCRESULTFILE, cor_calc_method))
  }
  colnames(COEFFRESULTS) <- generate_prcc_results_header(MEASURES)
  rownames(COEFFRESULTS) <- PARAMETERS
  return(COEFFRESULTS)
}
